﻿using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using System.Text;
using IdeaBlade.EntityModel;

// This sample uses Employee and Territory from the NorthwindIB database and shows two ways to work with
// many-to-many mappings with payload.
//
// In a many-to-many mapping with payload the mapping table is allowed to hold other information. The mapping table is mapped to 
// an Entity and managed like any other Entity in the system. This sample uses the EmployeeTerritory 
// table. To create a new mapping, you explicitly create a new EmployeeTerritory and assign the Employee 
// and Territory to it like employeeTerritory.Employee = employee and employeeTerritory.Territory = territory.
//
//   This sample does not save to the database, but you can do so easily by caling 
//   _entityManager.SaveChangesAsync();

namespace SilverlightConsole {
  public class CodeSamples {
    public CodeSamples() {
    }

    public void Start() {
      Coroutine.Start(Samples);
    }

    private IEnumerable<INotifyCompleted> Samples() {
      SLConsole.WriteLine("Note: The server may take a few seconds to initialize.\n");

      // Because Silverlight is asynchronous, we use the coroutine syntax to call these methods in order
      // Note that coroutines are not a part of nor are necessary for performing operations with entities in a many-to-many
      // relationship. 
      // If you're not familiar with coroutines, please disregard the "yield return's" and focus on the logic of the methods below.

      // The next three samples use the EmployeeTerritory mapping table
      yield return EmployeeTerritoryNavigationWithPayload();
      yield return AddTerritoryWithPayload();
      yield return DeleteTerritoryWithPayload();

      yield return Coroutine.Return(null);
    }

    // The three sample methods below are where the "many-to-many" operations take place.
    private INotifyCompleted EmployeeTerritoryNavigationWithPayload() {
      SLConsole.WriteLine("EmployeeTerritoryNavigationWithPayload Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories so we can iterate over them below
      var queryOp = _entityManager.Employees
        .Include("EmployeeTerritories.Territory")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      queryOp.Completed += (s, e) => {
        var employee = queryOp.Result;
        // employee.employeeTerritories is available immediately because it was cached by .Include("EmployeeTerritories.Territories")
        var employeeTerritories = employee.EmployeeTerritories.ToList();

        SLConsole.WriteLine("Employee " + employee.FirstName + employee.LastName
          + " is assigned to " + employee.EmployeeTerritories.Count().ToString() + " territories:");

        // employeeTerritory.Territory is available immediately because it was cached by .Include("EmployeeTerritories.Territories")
        foreach (var employeeTerritory in employeeTerritories) {
          SLConsole.WriteLine(employeeTerritory.Territory.TerritoryDescription.Trim());
        }
        SLConsole.WriteLine();
      };
      return queryOp;
    }

    private INotifyCompleted AddTerritoryWithPayload() {
      Territory territoryToBeAdded;
      SLConsole.WriteLine("AddTerritoryWithPayload Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories
      var queryOp = _entityManager.Employees
        .Include("EmployeeTerritories.Territory")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      queryOp.Completed += (s, e) => {
        var employee = queryOp.Result;

        SLConsole.WriteLine("Employee " + employee.FirstName + " " + employee.LastName
          + " is currently assigned to " + employee.EmployeeTerritories.Count().ToString() + " territories.");

        // Create a new Territory
        territoryToBeAdded = new Territory();
        territoryToBeAdded.TerritoryDescription = "Oakland";
        territoryToBeAdded.RegionID = 2;
        // Remember to add it to the EntityManager
        _entityManager.AddEntity(territoryToBeAdded);

        SLConsole.WriteLine("Assigning newTerritory [" + territoryToBeAdded.TerritoryDescription
          + "] to " + employee.FirstName + " " + employee.LastName);

        // Create a new EmployeeTerritory (the mapping entity/table)
        EmployeeTerritory newEmployeeTerritory = new EmployeeTerritory();
        newEmployeeTerritory.Employee = employee;
        newEmployeeTerritory.Territory = territoryToBeAdded;
        // Remember to add it to the EntityManager
        _entityManager.AddEntity(newEmployeeTerritory);

        SLConsole.WriteLine("Now " + employee.FirstName + " " + employee.LastName
          + " is currently assigned to " + employee.EmployeeTerritories.Count() + " territories.");
        SLConsole.WriteLine();
      };
      return queryOp;
    }

    private INotifyCompleted DeleteTerritoryWithPayload() {
      SLConsole.WriteLine("DeleteTerritoryWithPayload Started...");
      _entityManager.Clear();

      // Get an Employee and include their Territories
      var queryOp = _entityManager.Employees
        .Include("EmployeeTerritories.Territory")
        .Where(e => e.EmployeeID == 1)
        .AsScalarAsync()
        .First();

      queryOp.Completed += (s, e) => {
        var employee = queryOp.Result;

        SLConsole.WriteLine("Employee " + employee.FirstName + " " + employee.LastName
          + "is currently assigned to " + employee.EmployeeTerritories.Count() + " territories.");

        // Get an EmployeeTerritory mapping that is to be deleted
        EmployeeTerritory employeeTerritoryToBeremoved = employee.EmployeeTerritories.First();

        SLConsole.WriteLine("Unassigning Territory [" + employeeTerritoryToBeremoved.Territory.TerritoryDescription.Trim()
          + "] from " + employee.FirstName + " " + employee.LastName);

        // Delete the EmployeeTerritory
        // Note that in this example, we did not delete any Territories, only the mapping between Employee and Territory
        employeeTerritoryToBeremoved.EntityAspect.Delete();

        SLConsole.WriteLine("Now " + employee.FirstName + " " + employee.LastName
          + " is currently assigned to " + employee.EmployeeTerritories.Count() + " territories.");
        SLConsole.WriteLine();
      };
      return queryOp;
    }

    private NorthwindIBEntities _entityManager = new NorthwindIBEntities();
  }
}
